@@ -1,21 +1,12 @@
 /* vi:set ts=8 sts=4 sw=4:
  *
  * VIM - Vi IMproved		by Bram Moolenaar
- *				GUI support by Robert Webb
+ *				Amiga GUI support by Michael Nielsen
  *
  * Do ":help uganda"  in Vim to read copying and usage conditions.
  * Do ":help credits" in Vim to see a list of people who contributed.
  *
- * Windows GUI.
- *
- * GUI support for Microsoft Windows.  Win32 initially; maybe Win16 later 
- *
- * George V. Reilly <gvr@halcyon.com> wrote the original Win32 GUI.
- * Robert Webb reworked it to use the existing GUI stuff and added menu,
- * scrollbars, etc.
- *
- * Note: Clipboard stuff, for cutting and pasting text to other windows, is in
- * os_win32.c.	(It can also be done from the terminal version).
+ * Amiga GUI header file.
  */
 
 #if !defined(__GUI_AMIGA__H)
@@ -46,7 +37,7 @@ enum event {
 
 struct MyMenuItem {
     struct MenuItem menuItem;
-    struct GuiMenu *guiMenu;
+    struct VimMenu *guiMenu;
 };
 
 union myMenuItemUnion {
@@ -54,10 +45,6 @@ union myMenuItemUnion {
     struct MyMenuItem myMenuItem;
 };
 
-
-#include "vim.h"
-
-
 #endif /* USE_GUI_AMIGA*/
 #endif /* __GUI_AMIGA__H */
 
